#include <errno.h>
#include <stdio.h>
#include <stdio_ext.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <string.h>
#include <sys/wait.h>
#include <termios.h>
#include <stdarg.h>

#include "../include/AnalysisModuleInterface.h"
#include "AnalysisResultManager.h"
#include "PStringToken.h"
#include "PString.h"
#include "SplintAnalysisInfo.h"
#include "SplintAnalysis.h"

char pDefaultOption[] = { "OptionName$OptionType$OptionValue" };

extern "C" bool GetModuleInfo( CAnalysisModuleInfo* pInfo )
{ 
	pInfo->m_uiId = MODULE_ID;
	memcpy( pInfo->m_szName , MODULE_NAME , strlen( MODULE_NAME ) );
	pInfo->m_uiRevisionNo = MODULE_REVISION_NO; 

	pInfo->m_uiInputId = IOP_SOURCE_FILES;
	memcpy( pInfo->m_szInputName , PIN_NAME_SOURCE_FILE , strlen( PIN_NAME_SOURCE_FILE ) );
	// pInfo->m_szInputDescription  

	pInfo->m_uiOutputId = IOP_ANALYSIS_RESULT;
	memcpy( pInfo->m_szOutputName , PIN_NAME_ANALYSIS_RESULT , strlen( PIN_NAME_ANALYSIS_RESULT ) );
	// pInfo->m_szOutputDescription 

	return true;
}

extern "C" void	Initialize( CAnalysisNotifyInterface* pInterface , CModuleRelayInterface* pRelayInterface )
{
	g_ModuleErrorManager.Initialize( MODULE_ID , pInterface , pRelayInterface );
	g_AnalysisResultManager.Initialize( MODULE_ID , pInterface , pRelayInterface );
}

extern "C" bool StartProcess( CInOutPinBase* pInputPin )
// extern "C" bool StartProcess( CAnalysisNotifyInterface* pInterface , CInOutPinBase* pInputPin , CInOutPinBase** pOutputPin )
{
	int nFd[ 2 ];
	int nErrFd[ 2 ];
	
	pid_t pidChild;
	
	printf( "in Extern StartProcess\n" );

	if( pipe( nFd ) == -1 )
	{
		perror( "Failed to create the pipe\n" );
		return 1;
	}

	if( pipe( nErrFd ) == -1 )
	{
		perror( "Failed to create the error pipe\n" );
	}	
	
	pidChild = fork();

	if( pidChild == -1 )
	{
		perror( "Failed to fork\n" );
		return 1;
	}
	
	if( pidChild  > 0 )
	{
		CSourceFilePin* pSourceFilePin = ( CSourceFilePin* )pInputPin;

		char szDefaultCommand[]	= { "./splint " };
		char pszReplaceString[]	= { " " };
		char pszTargetString[]	= { "," };

		int nChildPid			= 0;
		int nSourceFileCount		= pSourceFilePin->m_nFileCount;
		int nDefaultCommandLength	= strlen(szDefaultCommand ); 
		int nSourceFileNameFullLength	= strlen( pSourceFilePin->m_pszFileName );

		PString strString( nDefaultCommandLength + nSourceFileNameFullLength + 1 );

		dup2( nFd[ 1 ] , STDOUT_FILENO );
		close( nFd[ 0 ] );
		close( nFd[ 1 ] );
		
		dup2( nErrFd[ 1 ] , STDERR_FILENO );
		close( nErrFd[ 0 ] );
		close( nErrFd[ 1 ] ); 

		strncpy( strString.GetString() , szDefaultCommand , nDefaultCommandLength );
		strncpy( strString.GetString() + nDefaultCommandLength , pSourceFilePin->m_pszFileName , nSourceFileNameFullLength ); 
	
		strString.StringReplace( pszTargetString , pszReplaceString );

		// printf( "Command = %s|\n" , strString.GetString() );
		system( strString.GetString() );

		close( STDOUT_FILENO );
		close( STDERR_FILENO );

		pidChild = wait( NULL );
		if( pidChild != -1 )
		{	
			// wait success
		}
	}
	else
	{
		close( nFd[ 1 ]	);
		close( nErrFd[ 1 ] ); 

		ProcessReadData( nFd[ 0 ] );

		// error 일때의 ProcessReadData를 만들어서 처리하면 되겠다.
		ProcessReadErrorData( nErrFd[ 0 ] );
		// ProcessReadData( nErrFd[ 0 ] ); 
	}	

	g_AnalysisResultManager.ProcessBufferData( true );
	// printf( "BufferUsedLength = %d\n" , ( g_AnalysisResultManager.GetBuffer() )->GetBufferUsedLength() );

	return 0;
}

extern "C" bool StopProcess()
{
	char pKillCommand[] = { "kill -9" };
	char pCommand[ 32 ];

	memset( pCommand , 0 , sizeof( pCommand ) );

	pid_t nPid = g_AnalysisResultManager.GetModulePid();

	sprintf( pCommand , "%s %d" , pKillCommand , nPid );

	system( pCommand );
}

void ProcessReadData( int nFd )
{
	int nReadSize = 0;
	char pBuffer[ MAX_BUFFER_SIZE ];

	memset( pBuffer , 0 , MAX_BUFFER_SIZE );		

	while( 0 != ( nReadSize = read( nFd , pBuffer , MAX_BUFFER_SIZE ) ) )
	{
		// printf( "---- pBuffer Content = %s nReadSize = %d\n" , pBuffer , nReadSize );
		// printf( "%s" , pBuffer );
		g_AnalysisResultManager.AddAnalysisResult( pBuffer , nReadSize );
		g_AnalysisResultManager.ProcessBufferData();
		
		nReadSize = 0;
		memset( pBuffer , 0 , MAX_BUFFER_SIZE );
	}
	g_AnalysisResultManager.ProcessBufferData();

	close( nFd );
}

void ProcessReadErrorData( int nFd )
{
	int nReadSize = 0;
	char pBuffer[ MAX_BUFFER_SIZE ];

	memset( pBuffer , 0 , MAX_BUFFER_SIZE );		

	while( 0 != ( nReadSize = read( nFd , pBuffer , MAX_BUFFER_SIZE ) ) )
	{
		// printf( "In Dll pBuffer Content = %s nReadSize = %d\n" , pBuffer , nReadSize );
		// printf( "%s" , pBuffer );
		g_ModuleErrorManager.AddAnalysisResult( pBuffer , nReadSize );
		g_ModuleErrorManager.ProcessBufferErrorData();
		
		nReadSize = 0;
		memset( pBuffer , 0 , MAX_BUFFER_SIZE );
	}
	g_ModuleErrorManager.ProcessBufferErrorData();

	close( nFd );
}

/* bool MemoryAllocation( char** pMemoryBuffer , int nSize )
{
	*pMemoryBuffer = ( char * )malloc( nSize );
	if( NULL == ( *pMemoryBuffer ) )
	{
		return false;
	}

	return true;

} */

/* void StringReplace( char* pStringBuffer , char* pTargetString , char* pReplaceString  )
{
	int i = 0;

	for( i = 0 ; ( i < strlen( pStringBuffer ) ) && ( '\0' != ( *pStringBuffer ) ) ; i++ )
	{
		if( 0 == strncmp( pStringBuffer + i , pTargetString , strlen( pTargetString ) ) )
		{
			memcpy( pStringBuffer + i , pReplaceString , strlen( pReplaceString ) );

			i += ( strlen( pReplaceString ) - 1 );
		}
	}
} */
